/* Program to generate IP data file */
#define ProgName "ipf2rax"
#define ProgInfo "Generate R-AXIS Image File for fiber-specimen"
/*                                                         8/26/2002  */
/*         for ANSI C                                                 */

/*
 * Note : before v0.13, coordinates in xyz file are assumed to be the system
 *   where c-axis is along fiber axis and a-axis is in xz-plane.
 *   Now (since v0.13), c-axis is the same as above, but b-axis is in yz-plane.
 *   Above axes are that when c- is fiber axis.
 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.20_6/25/2009"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "libxray.h"

#define MAXARG 26
#define MINARG 13
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define FLAT 0
#define CURVE_V 1
#define CURVE_H 2 /* NOT USED */
#define BACK 3 /* NOT USED */
#define MAXATOM 100000
#define REPNUM_DEFAULT 360
#define REPDIV_MIN 36
#define QLIM_DEFAULT 0.02

struct atomPos {
	char atom[4];
	long int atomType;
	double val[3];
};

#define SIZE_OF_ATOM_POS sizeof(struct atomPos)

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	char rotate3Dbl();
	char rotate3Long();
	char sinThetaFlat();
	char sinThetaCurveV();
	char (*sinThetaf)();
	double lauefunc();
	double lauefunc2();
	double (*lauef)();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2, double *arg3);
	char rotate3Dbl(double *arg1, double *arg2, double *arg3);
	char rotate3Long(long int *arg1, long int *arg2, long int *arg3);
	char sinThetaFlat(const double *x, const double *z, double *sinThetaSq, double *qval_p2, double *zeta, double *corr);
	char sinThetaCurveV(const double *x, const double *z, double *sinThetaSq, double *qval_p2, double *zeta, double *corr);
	char (*sinThetaf)(const double *x, const double *z, double *sinThetaSq, double *qval_p2, double *zeta, double *corr);
	double lauefunc(double *ind, double *size);
	double lauefunc2(double *ind, double *size);
	double (*lauef)(double *ind, double *size);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr, *datPtr0;
	long int i, j, ii, jj, ipHeaderSize, ipHeaderSize2VER2, lineNum,
	     recordLengthCreate, ipHeaderSizeMin, pos, atomNum,
	     cellNumA, cellNumB, cellNumC, repNum, tmpFlag, flushWidth0, flushWidth;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], file3[FNAMELEN], cpuIntelFlag, confirmBuffer[8],
	     inputBuffer[20], argNum, arg0[4], verboseFlag, shiftBit, xsizDoneFlag,
	     zsizDoneFlag, xpxlDoneFlag, zpxlDoneFlag, camrDoneFlag, lambdaDoneFlag,
	     drtFlag, relFlag, laueFlag, magDoneFlag, cameraTypeDoneFlag, cameraTypeFlag,
	     fibType, fibDoneFlag, dim_aFlag, dim_bFlag, dim_cFlag, ang_aFlag,
	     ang_bFlag, ang_cFlag, repDivFlag, repNumFlag, debug, qlimFlag,
	     fbufindex[ATOMTABLESIZE], anomalousFlag, useRefFlag, ipHeaderSwapFlag,
	     tiltFlag, ignoreSubPeaksFlag;
	double rato, arg1, arg2, arg3, intensityDbl, mag, fact1x, fact1z, fact2,
	       zDist, xDist, qval2, corr, ind_h, ind_k, ind_l, lTmp0, qval_p, qval_p2,
	       f_real, f_imag, phi, sf, ss, dim_a, dim_b, dim_c, cosPhi, sinPhi,
	       ang_A, ang_B, ang_C, cosA, cosB, cosC, matrix[9], v_fact, sinA,
	       laue_h, laue_k, laue_l, lambda, lambda2, maxVal, repDiv, i_repDiv, divWidth,
	       sinThetaSq, xi, qlim, fbuf[ATOMTABLESIZE][3], tiltAngle, tiltAxis,
	       i_lambda, i_lambda2, cosXi, sinXi, cosD, sinD, cosE, sinE,
	       cf_h_c, cf_h_s, cf_h_0, cf_k_c, cf_k_s, cf_k_0, cf_l_s, cf_l_0,
	       cf_h_cx, cf_h_sx, cf_h_0z, cf_k_cx, cf_k_sx, cf_k_0z, cf_l_sx, cf_l_0z,
#ifdef PARANOID	/* this is not necessary, for full rotation about z-axis done. */
	       cf_h_cy, cf_h_sy, cf_k_cy, cf_k_sy, cf_l_cy, cf_l_c, q_x, q_y,
#endif
	       cellNumA_P, cellNumB_P, cellNumC_P;
	float *datPtrF;
	struct ipHead ipHeader;
	time_t timeNow;
	FILE *fp1, *fp2, *fp3;
	struct atomPos *xyz;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	xsizDoneFlag = zsizDoneFlag = xpxlDoneFlag = zpxlDoneFlag = camrDoneFlag
	 = lambdaDoneFlag = drtFlag = relFlag = laueFlag = magDoneFlag
	 = cameraTypeDoneFlag = fibDoneFlag = dim_aFlag = dim_bFlag = dim_cFlag
	 = ang_aFlag = ang_bFlag = ang_cFlag = repDivFlag = repNumFlag
	 = debug = qlimFlag = anomalousFlag = useRefFlag = tiltFlag
	 = ignoreSubPeaksFlag = 0;
	lambda = lambda2 = 0.0;
	repDiv = 0.0;
	i_repDiv = 1.0;	/* dummy */
	sf = 0.0;
	tmpFlag = 0;
	cameraTypeFlag = FLAT;
	ipHeaderSwapFlag = 0;
	verboseFlag = 1;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	qval2 = 0.0;
	lineNum = 1;
	mag = 1.0;
	fibType = 2;
	flushWidth0 = 0;
	repNum = REPNUM_DEFAULT;
	qlim = QLIM_DEFAULT;
	tiltAngle = 0.0;
	tiltAxis = 0.0;
	initXrayAtom();
	for (i=0; i<ATOMTABLESIZE; i++) fbufindex[i] = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strncpy(file1, argv[0], FNAMELEN-1);
		strncpy(file2, argv[argc-1], FNAMELEN-1);
		for (i=1; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3) || !strncmp(argv[i], "-R:", 3)) {
				if (useRefFlag) {
					fprintf(stderr, "Too much REFFILEs.\n");
					exit(EXIT_FAILURE);
				}
				if (strlen(argv[i]) < 4) {
					fprintf(stderr, "\"REFFILE\" must be provided.\n");
					exit(EXIT_FAILURE);
				}
				strncpy(file3, argv[i]+3, FNAMELEN-1);
				useRefFlag = 1;
				if (!strncmp(argv[i], "-R:", 3)) useRefFlag = 2;
			}
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* Interactive ? */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you like to use Reference File to get parameters for New File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			useRefFlag = 1;
			fprintf(stdout, " Enter Reference Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				exit(EXIT_FAILURE);
			}
			do {
				fprintf(stdout, " Do you like to use Whole Image Size[W] or Largest Quadrant[Q] ? : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'W') && (toupper(confirmBuffer[0]) != 'Q'));
			if (toupper(confirmBuffer[0]) == 'Q') useRefFlag = 2;
		}
	}
	
/* Define ipHeader and Initialize */
	if (useRefFlag) {	/* Read REFFILE */
		if ((fp3 = fopen(file3, "rb")) == NULL) {
			fprintf(stderr, " Reference File Open Error.\n");
			exit(EXIT_FAILURE);
		}
		fread(&ipHeader, ipHeaderSize, 1, fp3);
		if (ftell(fp3) != ipHeaderSize) {
			fprintf(stderr, " Reference File Read Error.\n");
			fclose(fp3);
			exit(EXIT_FAILURE);
		}
		fclose(fp3);
		/* File Format Check */
		ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
		/* ipHeader.rlng is assumed to be less than 65536;
		   if each data has 16-bits, width of IP should less than 32768 pixels. */
		if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
			if (verboseFlag) {
				fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
				do {
					fprintf(stdout, "  Is it true ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
				if (toupper(confirmBuffer[0]) == 'N') {
					fprintf(stderr, " Unable to continue. Stop.\n");
					exit(EXIT_FAILURE);
				}
				fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
			}
		}
		/* Swap Header */
		if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
		/* Get ipHeader Information and Check */
		if (ipHeader.zpxl != ipHeader.rnum) {
			ipHeader.zpxl = ipHeader.rnum;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in Reference File has been modified.\n");
		}
		if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
			ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in Reference File has been modified.\n");
		}
		if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)
		 || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)
		 || (ipHeader.xsiz <= 0.0) || (ipHeader.zsiz <= 0.0)
		 || (ipHeader.camr <= 0.0) || (ipHeader.wlng <= 0.0)) {
			fprintf(stderr, " Reference File Format is Incorrect.\n");
			exit(EXIT_FAILURE);
		}
		if (useRefFlag == 2) {
			if ((ipHeader.drtx <= 0.0) || (ipHeader.drtx > ipHeader.xpxl)
			 || (ipHeader.drtx <= 0.0) || (ipHeader.drtx > ipHeader.xpxl)) {
				fprintf(stderr, " Reference File Format is Incorrect.\n");
				exit(EXIT_FAILURE);
			}
			ipHeader.xpxl = (long)ceil(MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx));
			ipHeader.zpxl = (long)ceil(MAXEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz));
			ipHeader.drtx = 0.0;
			ipHeader.drtz = 0.0;
		}
		lambda = ipHeader.wlng;
		/* Clean Header */
		cleanIpHeader(&ipHeader);
		ipHeader.rato = rato;
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcpy(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, "/");
		strcat(ipHeader.memo, ipHeader.date);
	}
	else {	/* Do not use REFFILE */
		memset(&ipHeader, 0, ipHeaderSize);
		strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
		strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
		strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
		strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
		ipHeader.alng = 10.0; /* a-axis length in A */
		ipHeader.blng = 10.0; /* b-axis length in A */
		ipHeader.clng = 10.0; /* c-axis length in A */
		ipHeader.alfa = 90.0; /* angle alpha */
		ipHeader.beta = 90.0; /* angle beta */
		ipHeader.gamm = 90.0; /* angle gamma */
		strcpy(ipHeader.spgr, "P23"); /* Space Group */
		ipHeader.mosc = 0.3; /* Mozaic Spread */
		strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
		strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
		strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
		ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
		strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
		strcpy(ipHeader.filt, "Ni"); /* Kb filter */
		ipHeader.vltg = 40.0; /* X-ray tube voltage */
		ipHeader.crnt = 100.0; /* X-ray tube current */
		strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
		strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
		ipHeader.cyld = 0; /* Camera type */
		ipHeader.weis = 0.0; /* weisenberg oscillation */
		strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
		strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
		ipHeader.phi0 = 0.0; /* phi0 */
		ipHeader.phis = 0.0; /* phi start */
		ipHeader.phie = 0.0; /* phi end */
		ipHeader.oscn = 0; /* Frequency of Oscillation */
		ipHeader.drtx = 0.0; /* Direct Position X */
		ipHeader.drtz = 0.0; /* Direct position Y */
		ipHeader.omga = 0.0; /* omega in theta */
		ipHeader.fkai = 0.0; /* xai */
		ipHeader.thta = 0.000; /* 2-theta */
		ipHeader.mu = 0.000; /* mu */
		ipHeader.ipst = 0; /* Line of start reading */
		ipHeader.ipnm = 1; /* Number of IP */
		ipHeader.rato = rato; /* Ratio of IP in High/Low */
		ipHeader.ft_1 = 0.0; /* fading time 1 */
		ipHeader.ft_2 = 0.0; /* fading time 2 */
		strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
		strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
		ipHeader.dr_x = 0; /* Data direction in horizontal */
		ipHeader.dr_z = 0; /* Data direction in vertical */
		ipHeader.drxz = 0; /* Data direction face/back */
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcat(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, "/");
		strcat(ipHeader.memo, ipHeader.date);
	}
	
/* Get Input Filename and Reading */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Filter Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "r")) == NULL
	 && (fp1 = fopen(strcat(file1, ".xyz"), "r")) == NULL) {
		fprintf(stderr, " Input File Open Error.\n");
		exit(EXIT_FAILURE);
	}
/* Reading XYZ File */
	fscanf(fp1, "%s", inputBuffer);
	if (!isdigitEXT(inputBuffer)) {
		fprintf(stderr, " Error. Irregal Input File Format.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	atomNum = atol(inputBuffer);
	if (atomNum <= 0 || atomNum > MAXATOM) {
		fprintf(stderr, " Error. Irregal Atom Number.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	fseek(fp1, 0, SEEK_SET);
	do {
		if (fgetc(fp1) == '\n') tmpFlag++;
	} while (tmpFlag < 2);
	if ((xyz = malloc(SIZE_OF_ATOM_POS*atomNum)) == NULL) {
		fprintf(stderr, " Momery Allocation Error. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	for (i=0; i<atomNum; i++) {
		if (fscanf(fp1, "%s", xyz[i].atom) == EOF) {
			fprintf(stderr, " Error in reading input File. Exit.\n");
			free(xyz);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		for (j=0; j<3; j++) {
			if (fscanf(fp1, "%lf", &(xyz[i].val[j])) == EOF) {
				fprintf(stderr, " Error in reading input File. Exit.\n");
				free(xyz);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		xyz[i].atomType = checkXrayAtom(xyz[i].atom);
		if (xyz[i].atomType == -1) {
			fprintf(stderr, " Error. Irregal or Not Supported Atom Specification : %s\n", xyz[i].atom);
			free(xyz);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fbufindex[xyz[i].atomType] = 1;
	}
	fclose(fp1);
	
/* Conversion Parameter */
	if (verboseFlag == 3) {
		fprintf(stdout, "1. Information about structure.\n");
		do {
			fprintf(stdout, " Enter length of a-axis[A] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			dim_a = atof(inputBuffer);
			if (dim_a <= 0.0) continue;
			dim_aFlag = 1;
		} while (!dim_aFlag);
		do {
			fprintf(stdout, " Enter length of b-axis[A] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			dim_b = atof(inputBuffer);
			if (dim_b <= 0.0) continue;
			dim_bFlag = 1;
		} while (!dim_bFlag);
		do {
			fprintf(stdout, " Enter length of c-axis[A] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			dim_c = atof(inputBuffer);
			if (dim_c <= 0.0) continue;
			dim_cFlag = 1;
		} while (!dim_cFlag);
		do {
			fprintf(stdout, " Enter Angle of A[deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ang_A = atof(inputBuffer);
			if (ang_A <= 0.0 || ang_A >= 180.0) continue;
			ang_aFlag = 1;
		} while (!ang_aFlag);
		do {
			fprintf(stdout, " Enter Angle of B[deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ang_B = atof(inputBuffer);
			if (ang_B <= 0.0 || ang_B >= 180.0) continue;
			ang_bFlag = 1;
		} while (!ang_bFlag);
		do {
			fprintf(stdout, " Enter Angle of C[deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ang_C = atof(inputBuffer);
			if (ang_C <= 0.0 || ang_C >= 180.0) continue;
			ang_cFlag = 1;
		} while (!ang_cFlag);
		do {
			fprintf(stdout, " Enter Fiber-Axis [a/b/c] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'A') && (toupper(confirmBuffer[0]) != 'B') && (toupper(confirmBuffer[0]) != 'C'));
		if (toupper(confirmBuffer[0]) == 'A') fibType = 0;
		else if (toupper(confirmBuffer[0]) == 'B') fibType = 1;
		else fibType = 2;
		do {
			fprintf(stdout, " Do you want to assume tilting ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter direction of tilting axis in %s plane [deg.].\n",
				        (fibType ? (fibType==1 ? "C* and A*" : "A* and B*") : "B* and C*"));
				fprintf(stdout, "   Angle is measured from %s*-axis to %s*-axis direction : ",
				        (fibType ? (fibType==1 ? "C*" : "A*") : "B*"), (fibType ? (fibType==1 ? "A*" : "B*") : "C*"));
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				tiltAxis = atof(inputBuffer);
				if (tiltAxis) continue;
				tiltFlag = 1;
			} while (!tiltFlag);
			tiltFlag = 0;
			do {
				fprintf(stdout, "  Enter tilt angle [deg.] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				tiltAngle = atof(inputBuffer);
				if (tiltAngle) continue;
				tiltFlag = 1;
			} while (!tiltFlag);
		}
		do {
			fprintf(stdout, " Is input .xyz file Fractional[F] or Absolute[A] coordinates ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'F') && (toupper(confirmBuffer[0]) != 'A'));
		if (toupper(confirmBuffer[0]) == 'F') relFlag = 1;
		fprintf(stdout, "2. Option for calculation.\n");
		do {
			fprintf(stdout, " Enter Number of Cell along a-axis : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			cellNumA = atol(inputBuffer);
			if (cellNumA < 1) continue;
			laueFlag = 1;
		} while (!laueFlag);
		laueFlag = 0;
		do {
			fprintf(stdout, " Enter Number of Cell along b-axis : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			cellNumB = atol(inputBuffer);
			if (cellNumB < 1) continue;
			laueFlag = 1;
		} while (!laueFlag);
		laueFlag = 0;
		do {
			fprintf(stdout, " Enter Number of Cell along c-axis : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			cellNumC = atol(inputBuffer);
			if (cellNumC < 1) continue;
			laueFlag = 1;
		} while (!laueFlag);
		do {
			fprintf(stdout, " Way of division for average, after equal-step[S] or equal-length[L] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'S') && (toupper(confirmBuffer[0]) != 'L'));
		if (toupper(confirmBuffer[0]) == 'S') {
			do {
				fprintf(stdout, "  Equal-Step. Enter number of division for each averaging : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				repNum = atof(inputBuffer);
				if (repNum <= 0.0) continue;
				repNumFlag = 1;
			} while (!repNumFlag);
		}
		else {
			do {
				fprintf(stdout, "  Equal-Length. Enter length[1/A] of each division for averaging : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				repDiv = atof(inputBuffer);
				if (repDiv <= 0.0) continue;
				repDivFlag = 1;
			} while (!repDivFlag);
		}
		do {
			fprintf(stdout, " Take into account of anomalous scattering factor ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') anomalousFlag = 1;
		fprintf(stdout, "3. Information about output file.\n");
		do {
			fprintf(stdout, " Camera is Flat[F] , Curved[C] or Flat(Back)[B] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'F') && (toupper(confirmBuffer[0]) != 'C'));
		if (toupper(confirmBuffer[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else {
			ipHeader.cyld = 1;
			fprintf(stdout, "  Camera is Curved, with Cylinder axis vertical.\n");
			cameraTypeFlag = CURVE_V;
		}
		if (useRefFlag) {
			fprintf(stdout, "Reference File has following Values :\n");
			fprintf(stdout, " ImageSize is %ld [pixel] x %ld [pixel], WaveLength is %10.6f [A]\n", ipHeader.xpxl, ipHeader.zpxl, ipHeader.wlng);
			fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
			if (useRefFlag == 2) {
				fprintf(stdout, " New ImageSize is %ld [pixel] x %ld [pixel]\n",
				 (long)ceil(MAXEXT(ipHeader.drtx, ipHeader.xpxl-ipHeader.drtx)),
				 (long)ceil(MAXEXT(ipHeader.drtz, ipHeader.zpxl-ipHeader.drtz)));
				ipHeader.drtx = 0.0;
				ipHeader.drtz = 0.0;
			}
		}
		do {
			fprintf(stdout, " Enter Size of Image in X-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				xpxlDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xpxl = atol(inputBuffer);
			if (ipHeader.xpxl <= 0 || ipHeader.xpxl > MAXWIDTH) continue;
			xpxlDoneFlag = 1;
		} while (!xpxlDoneFlag);
		do {
			fprintf(stdout, " Enter Size of Image in Z-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				zpxlDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zpxl = atol(inputBuffer);
			if (ipHeader.zpxl <= 0 || ipHeader.zpxl > MAXHEIGHT) continue;
			zpxlDoneFlag = 1;
		} while (!zpxlDoneFlag);
		do {
			fprintf(stdout, " Enter PixelSize in X-direction [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				xsizDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xsiz = atof(inputBuffer);
			if (ipHeader.xsiz <= 0.0) continue;
			xsizDoneFlag = 1;
		} while (!xsizDoneFlag);
		do {
			fprintf(stdout, " Enter PixelSize in Z-direction [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				zsizDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zsiz = atof(inputBuffer);
			if (ipHeader.zsiz <= 0.0) continue;
			zsizDoneFlag = 1;
		} while (!zsizDoneFlag);
		do {
			fprintf(stdout, " Is direct position in ( 0, 0 )[A] or ( 0.5, 0.5 )[B] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'A') && (toupper(confirmBuffer[0]) != 'B'));
		if (toupper(confirmBuffer[0]) == 'B') drtFlag = 1;
		do {
			fprintf(stdout, " Enter Camera Length [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				camrDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.camr = atof(inputBuffer);
			if (ipHeader.camr <= 0.0) continue;
			camrDoneFlag = 1;
		} while (!camrDoneFlag);
		do {
			fprintf(stdout, " Enter WaveLength [A]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				lambdaDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			lambda = ipHeader.wlng = atof(inputBuffer);
			if (ipHeader.wlng <= 0.0) continue;
			lambdaDoneFlag = 1;
		} while (!lambdaDoneFlag);
		do {
			fprintf(stdout, " Enter Magnification to Generate Image : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			mag = atof(inputBuffer);
			if (mag <= 0.0) continue;
			magDoneFlag = 1;
		} while (!magDoneFlag);
		fprintf(stdout, " Pixels with q smaller than %6.4f are ignored while intensity normalization.\n", QLIM_DEFAULT);
		do {
			fprintf(stdout, "  Do you want to change this value ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(xyz);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, " Enter Critical q-value[1/A] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				qlim = atof(inputBuffer);
				if (qlim < 0.0) continue;
				qlimFlag = 1;
			} while (!qlimFlag);
		}
	}
	else {
		for (i=1; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3) || !strncmp(argv[i], "-R:", 3)) continue;
			argNum = splitArg(argv[i], arg0, &arg1, &arg2, &arg3);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-r")) relFlag++;
				else if (!strcmp(arg0, "-aa")) {
					fibDoneFlag++;
					fibType = 0;
				}
				else if (!strcmp(arg0, "-bb")) {
					fibDoneFlag++;
					fibType = 1;
				}
				else if (!strcmp(arg0, "-cc")) {
					fibDoneFlag++;
					fibType = 2;
				}
				else if (!strcmp(arg0, "-sc")) drtFlag++;
				else if (!strcmp(arg0, "-d")) debug++;
				else if (!strcmp(arg0, "-an")) anomalousFlag++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-x")) {
					ipHeader.xpxl = dToLong(arg1);
					if (ipHeader.xpxl <= 0 || ipHeader.xpxl > MAXWIDTH) break;
					xpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					ipHeader.zpxl = dToLong(arg1);
					if (ipHeader.zpxl <= 0 || ipHeader.zpxl > MAXHEIGHT) break;
					zpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-lm")) {
					if (arg1 <= 0.0) break;
					ipHeader.wlng = lambda = arg1;
					lambdaDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					if (arg1 <= 0.0) break;
					mag = arg1;
					magDoneFlag++;
				}
				else if (!strcmp(arg0, "-a")) {
					if (arg1 <= 0.0) break;
					dim_a = arg1;
					dim_aFlag++;
				}
				else if (!strcmp(arg0, "-b")) {
					if (arg1 <= 0.0) break;
					dim_b = arg1;
					dim_bFlag++;
				}
				else if (!strcmp(arg0, "-c")) {
					if (arg1 <= 0.0) break;
					dim_c = arg1;
					dim_cFlag++;
				}
				else if (!strcmp(arg0, "-A")) {
					if (arg1 <= 0.0 || arg1 >= 180.0) break;
					ang_A = arg1;
					ang_aFlag++;
				}
				else if (!strcmp(arg0, "-B")) {
					if (arg1 <= 0.0 || arg1 >= 180.0) break;
					ang_B = arg1;
					ang_bFlag++;
				}
				else if (!strcmp(arg0, "-C")) {
					if (arg1 <= 0.0 || arg1 >= 180.0) break;
					ang_C = arg1;
					ang_cFlag++;
				}
				else if (!strcmp(arg0, "-nm")) {
					if (arg1 <= 0.0) break;
					repNum = dToLong(arg1);
					repNumFlag++;
				}
				else if (!strcmp(arg0, "-dy")) {
					if (arg1 <= 0.0) break;
					repDiv = arg1;
					repDivFlag++;
				}
				else if (!strcmp(arg0, "-cq")) {
					if (arg1 < 0.0) break;
					qlim = arg1;
					qlimFlag++;
				}
				else break;
			}
			else if (argNum == 3) {
				if (!strcmp(arg0, "-tl")) {
					tiltAxis = arg1;
					tiltAngle = arg2;
					tiltFlag++;
				}
				else break;
			}
			else if (argNum == 4) {
				if (!strcmp(arg0, "-la")) {
					cellNumA = dToLong(arg1);
					cellNumB = dToLong(arg2);
					cellNumC = dToLong(arg3);
					if (cellNumA < 1 || cellNumB < 1 || cellNumC < 1) break;
					laueFlag++;
				}
				else if (!strcmp(arg0, "-La")) {
					cellNumA = dToLong(arg1);
					cellNumB = dToLong(arg2);
					cellNumC = dToLong(arg3);
					if (cellNumA < 1 || cellNumB < 1 || cellNumC < 1) break;
					laueFlag++;
					ignoreSubPeaksFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(xyz);
			exit(EXIT_FAILURE);
		}
		if ((xpxlDoneFlag > 1) || (zpxlDoneFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (drtFlag > 1)
		 || (camrDoneFlag > 1) || (lambdaDoneFlag > 1) || (magDoneFlag > 1)
		 || (cameraTypeDoneFlag > 1) || (debug > 1) || (qlimFlag > 1)
		 || (dim_aFlag > 1) || (dim_bFlag > 1) || (dim_cFlag > 1)
		 || (ang_aFlag > 1) || (ang_bFlag > 1) || (ang_cFlag > 1)
		 || (relFlag > 1) || (fibDoneFlag > 1) || (tiltFlag > 1)
		 || (laueFlag > 1) || (repNumFlag > 1) || (lambdaDoneFlag > 1)
		 || (repDivFlag > 1) || (repDivFlag && repNumFlag)
		 || (anomalousFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(xyz);
			exit(EXIT_FAILURE);
		}
		if (((!xpxlDoneFlag || !zpxlDoneFlag || !xsizDoneFlag
		   || !zsizDoneFlag || !camrDoneFlag || !lambdaDoneFlag) && !useRefFlag)
		 || !cameraTypeDoneFlag
		 || !dim_aFlag || !dim_bFlag || !dim_cFlag
		 || !ang_aFlag || !ang_bFlag || !ang_cFlag
		 || !fibDoneFlag || !laueFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(xyz);
			exit(EXIT_FAILURE);
		}
		if (ipHeader.drtx > ipHeader.xpxl || ipHeader.drtz > ipHeader.zpxl) {
			fprintf(stderr, " Arguments out of Range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(xyz);
			exit(EXIT_FAILURE);
		}
	}
	ipHeader.drtx = ipHeader.drtz = drtFlag ? 0.5 : 0.0;
	qlim *= qlim;
	if (cameraTypeFlag == CURVE_V) {
		if ((ipHeader.xpxl-ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr > M_PI || ipHeader.drtx*ipHeader.xsiz/ipHeader.camr > M_PI) {
			fprintf(stderr, " Cylindrical camera is too wide.\n");
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(xyz);
			exit(EXIT_FAILURE);
		}
	}
	if (anomalousFlag) {
		for (i=0; i<ATOMTABLESIZE; i++) {
			if (fbufindex[i]) {
				if ((j = calcXrayAnFactor(&(fbuf[i][1]), &lambda, &i))) {
					if (j==1) fprintf(stderr, "Wavelength for Anomalous Scattering is unexpected.\n");
					else fprintf(stderr, "Unexpected error while calculating anomalous scattering factor.\n");
					free(xyz);
					exit(EXIT_FAILURE);
				}
			}
		}
	}
#if 0 /* not to treat anomalous scattering separately */
	else {
		for (i=0; i<ATOMTABLESIZE; i++) fbuf[i][1] = fbuf[i][2] = 0.0;
	}
#endif
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has %ld atoms.  Cell parameters are :\n", atomNum);
		fprintf(stdout, "   a = %9.4f[A] ,  b = %9.4f [A] ,  c = %9.4f [A],\n", dim_a, dim_b, dim_c);
		fprintf(stdout, "   A = %6.2f[deg.] ,  B = %6.2f [deg.] ,  C = %6.2f [deg.].\n", ang_A, ang_B, ang_C);
		if (tiltFlag) {
			fprintf(stdout, "  Fiber Axis is along %c, with tilting by %6.2f [deg.]\n", fibType+'A', tiltAngle);
			fprintf(stdout, "   around axis of %6.2f [deg.] direction in reciprocal basal plane.\n", tiltAxis);
		}
		else fprintf(stdout, "  Fiber Axis is along %c.\n", fibType+'A');
		fprintf(stdout, "  Crystal Size is %ld x %ld x %ld.\n", cellNumA, cellNumB, cellNumC);
		if (anomalousFlag) fprintf(stdout, "  Anomalous Scattering is taken into account.\n");
		fprintf(stdout, " ImageSize is %ld [pixel] x %ld [pixel], WaveLength is %10.6f [A]\n", ipHeader.xpxl, ipHeader.zpxl, ipHeader.wlng);
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		fprintf(stdout, " Centre is ( %7.1f x %7.1f )\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " Output Intensity is magnified by %10.2f times.\n", mag);
		if (cameraTypeFlag == FLAT) fprintf(stdout, " Camera is Flat.\n");
		else if (cameraTypeFlag == CURVE_V) fprintf(stdout, " Camera is Cylindrical along Z-direction.\n");
	}
	ipHeader.alng = dim_a;
	ipHeader.blng = dim_b;
	ipHeader.clng = dim_c;
	ipHeader.alfa = ang_A;
	ipHeader.beta = ang_B;
	ipHeader.gamm = ang_C;
	
/* change coordinate to c is along z-axis */
	if (fibType == 0) {
		rotate3Dbl(&ang_A, &ang_B, &ang_C);
		rotate3Dbl(&dim_a, &dim_b, &dim_c);
		rotate3Long(&cellNumA, &cellNumB, &cellNumC);
	}
	else if (fibType == 1) {	/* this may seems to be curious, but this is correct. */
		rotate3Dbl(&ang_C, &ang_B, &ang_A);
		rotate3Dbl(&dim_c, &dim_b, &dim_a);
		rotate3Long(&cellNumC, &cellNumB, &cellNumA);
	}
	ang_A *= DEG_RAD;
	ang_B *= DEG_RAD;
	ang_C *= DEG_RAD;
	cosA = cos(ang_A);
	cosB = cos(ang_B);
	cosC = cos(ang_C);
	sinA = sin(ang_A);
	for (i=0; i<atomNum; i++) {
		if (fibType == 0) rotate3Dbl(&(xyz[i].val[0]), &(xyz[i].val[1]), &(xyz[i].val[2]));
		else if (fibType == 1) rotate3Dbl(&(xyz[i].val[2]), &(xyz[i].val[1]), &(xyz[i].val[0]));
	}
	
/* Convert to fractional coordinates */
	v_fact = sqrt(1.0 - cosA*cosA - cosB*cosB - cosC*cosC + 2.0*cosA*cosB*cosC);
	if (!relFlag) {
		matrix[0] = sinA/dim_a/v_fact;
		matrix[1] = 0.0;
		matrix[2] = 0.0;
		matrix[3] = (cosA*cosB-cosC)/dim_b/sinA/v_fact;
		matrix[4] = 1.0/dim_b/sinA;
		matrix[5] = 0.0;
		matrix[6] = (cosC*cosA-cosB)/dim_c/sinA/v_fact;
		matrix[7] = -cosA/sinA/dim_c;
		matrix[8] = 1.0/dim_c;
		for (i=0; i<atomNum; i++) {
			xyz[i].val[2] = matrix[6]*xyz[i].val[0] + matrix[7]*xyz[i].val[1] + matrix[8]*xyz[i].val[2];
			xyz[i].val[1] = matrix[3]*xyz[i].val[0] + matrix[4]*xyz[i].val[1];
			xyz[i].val[0] = matrix[0]*xyz[i].val[0];
		}
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(flushWidth, ipHeaderSizeMin);
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	lambda2 = lambda*lambda;
	i_lambda = 1.0/lambda;
	i_lambda2 = 1.0/lambda2;
	fact1x = ipHeader.xsiz / ipHeader.camr;
	fact1z = ipHeader.zsiz / ipHeader.camr;
	fact2 = ipHeader.xsiz * ipHeader.zsiz / ipHeader.camr / ipHeader.camr / M_2PI;
	switch (cameraTypeFlag) {
		case FLAT:
			sinThetaf = sinThetaFlat;
			break;
		case CURVE_V:
			sinThetaf = sinThetaCurveV;
			break;
		default:
			break;
			/* NOT REACHED */
	}
	ipHeader.fext = mag; /* Exposure Time in min. */
	if (repDivFlag) i_repDiv = M_2PI/repDiv;
	cellNumA_P = cellNumA*M_PI;
	cellNumB_P = cellNumB*M_PI;
	cellNumC_P = cellNumC*M_PI;
	lauef = ignoreSubPeaksFlag ? lauefunc2 : lauefunc;
	
/* Conversion Parameters for Tilting */
	tiltAngle *= DEG_RAD;
	tiltAxis *= DEG_RAD;
	cosD = cos(tiltAngle);
	sinD = sin(tiltAngle);
	cosE = cos(tiltAxis);
	sinE = sin(tiltAxis);
	cf_h_cx = (-(cosA*cosB-cosC)*sinE+v_fact*cosE)*dim_a/sinA;
	cf_h_sx = (((cosA*cosB-cosC)*cosE+v_fact*sinE)*cosD/sinA + cosB*sinD)*dim_a;
	cf_h_0z = (-((cosA*cosB-cosC)*cosE+v_fact*sinE)*sinD/sinA + cosB*cosD)*dim_a;
	cf_k_cx = sinA*sinE*dim_b;
	cf_k_sx = (cosA*sinD - sinA*cosD*cosE) * dim_b;
	cf_k_0z = (sinA*sinD*cosE + cosA*cosD) * dim_b;
	cf_l_sx = sinD*dim_c;
	cf_l_0z = cosD*dim_c;
#ifdef PARANOID
	cf_h_cy = -cf_h_sx;
	cf_h_sy =  cf_h_cx;
	cf_k_cy = -cf_k_sx;
	cf_k_sy =  cf_k_cx;
	cf_l_cy = -cf_l_sx;
#endif
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			free(xyz);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		free(xyz);
		exit(EXIT_FAILURE);
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((datPtrF = malloc(ipHeader.xpxl*ipHeader.zpxl*sizeof(float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		free(xyz);
		exit(EXIT_FAILURE);
	}
	pos = 0;
	maxVal = 0.0;
	if (debug) {
		fprintf(stdout, "%6.2f%% ", 0.0);
		fflush(stdout);
	}
	if (anomalousFlag) {
		if (tiltFlag) {
			for (i=0; i<ipHeader.zpxl; i++) {
				zDist = (i+0.5-ipHeader.drtz) * fact1z;
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrF);
					free(xyz);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						}
						fclose(fp2);
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					xDist = (j+0.5-ipHeader.drtx) * fact1x;
					(*sinThetaf)(&xDist, &zDist, &sinThetaSq, &qval_p2, &lTmp0, &corr);
					lTmp0 *= i_lambda;	/* q_z */
					qval2 = sinThetaSq*i_lambda2;
#ifdef PARANOID	/* other sections can have this routine instead. */
					qval_p2 *= i_lambda2;
					qval_p = sqrt(qval_p2);
					q_y = -0.5*lambda*qval2;
					q_x = sqrt(qval_p2-q_y*q_y);
					cf_h_c = cf_h_cx * q_x + cf_h_cy * q_y;
					cf_h_s = cf_h_sx * q_x + cf_h_sy * q_y;
					cf_h_0 = cf_h_0z * lTmp0;
					cf_k_c = cf_k_cx * q_x + cf_k_cy * q_y;
					cf_k_s = cf_k_sx * q_x + cf_k_sy * q_y;
					cf_k_0 = cf_k_0z * lTmp0;
					cf_l_c = cf_l_cy * q_y;
					cf_l_s = cf_l_sx * q_x;
					cf_l_0 = cf_l_0z * lTmp0;
#else
					qval_p = sqrt(qval_p2)*i_lambda;
					cf_h_c = cf_h_cx * qval_p;
					cf_h_s = cf_h_sx * qval_p;
					cf_h_0 = cf_h_0z * lTmp0;
					cf_k_c = cf_k_cx * qval_p;
					cf_k_s = cf_k_sx * qval_p;
					cf_k_0 = cf_k_0z * lTmp0;
					cf_l_s = cf_l_sx * qval_p;
					cf_l_0 = cf_l_0z * lTmp0;
#endif
					if (repDivFlag) {
						repNum = dToLong(qval_p*i_repDiv);
						if (repNum < REPDIV_MIN) repNum = REPDIV_MIN;
					}
					divWidth = M_2PI / repNum;
					ss = 0.0;
					for (jj=0; jj<ATOMTABLESIZE; jj++) {
						if (fbufindex[jj]) {
							if (calcXrayFactor(&qval2, &jj, &(fbuf[jj][0]))) {
								fprintf(stderr, "Error in Calculating Scattering Factor.\n");
								free(datPtrF);
								free(xyz);
								if (flushWidth0 > 0) free(datPtr0);
								if (i) {
									ipHeader.zpxl = ipHeader.rnum = i;
									if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									fseek(fp2, 0, SEEK_SET);
									fwrite(&ipHeader, ipHeaderSize, 1, fp2);
									fprintf(stdout, " Output File is saved using available data.\n");
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									if (verboseFlag) {
										fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
										fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
									}
									fclose(fp2);
								}
								else {
									fclose(fp2);
									remove(file2);
									fprintf(stderr, " Output File is not created or removed.\n");
								}
								exit(EXIT_FAILURE);
							}
							fbuf[jj][0] += fbuf[jj][1];
						}
					}
					for (ii=0; ii<repNum; ii++) {
						xi = ii * divWidth;
						cosXi = cos(xi);
						sinXi = sin(xi);
						ind_h = cf_h_c * cosXi + cf_h_s * sinXi + cf_h_0;
						ind_k = cf_k_c * cosXi + cf_k_s * sinXi + cf_k_0;
#ifdef PARANOID
						ind_l = cf_l_c * cosXi + cf_l_s * sinXi + cf_l_0;
#else
						ind_l = cf_l_s * sinXi + cf_l_0;
#endif
						if ((laue_h = (*lauef)(&ind_h, &cellNumA_P)) == 0.0
						 || (laue_k = (*lauef)(&ind_k, &cellNumB_P)) == 0.0
						 || (laue_l = (*lauef)(&ind_l, &cellNumC_P)) == 0.0) continue;
						f_real = f_imag = 0.0;
						for (jj=0; jj<atomNum; jj++) {
							phi = M_2PI * (xyz[jj].val[0]*ind_h + xyz[jj].val[1]*ind_k + xyz[jj].val[2]*ind_l);
							cosPhi = cos(phi);
							sinPhi = sin(phi);
							f_real += fbuf[xyz[jj].atomType][0] * cosPhi - fbuf[xyz[jj].atomType][2] * sinPhi;
							f_imag += fbuf[xyz[jj].atomType][0] * sinPhi + fbuf[xyz[jj].atomType][2] * cosPhi;
						}
						sf = f_real * f_real + f_imag * f_imag;
						ss += sf * laue_h * laue_h * laue_k * laue_k * laue_l * laue_l;
					}
					ss *= (corr * fact2 * divWidth);
					datPtrF[pos++] = ss;
					if (ss > maxVal && qval2 > qlim) maxVal = ss;
					intensityDbl = ss * mag;
					datPtr[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				fflush(fp2);
				free(datPtr);
				if (debug) {
					fprintf(stdout, "\b\b\b\b\b\b\b\b");
					fprintf(stdout, "%6.2f%% ", (i+1)*100.0/ipHeader.zpxl);
					fflush(stdout);
				}
			}
		}
		else {
			for (i=0; i<ipHeader.zpxl; i++) {
				zDist = (i+0.5-ipHeader.drtz) * fact1z;
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrF);
					free(xyz);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						}
						fclose(fp2);
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					xDist = (j+0.5-ipHeader.drtx) * fact1x;
					(*sinThetaf)(&xDist, &zDist, &sinThetaSq, &qval_p2, &lTmp0, &corr);
					lTmp0 *= i_lambda;
					ind_l = lTmp0*dim_c;
					qval2 = sinThetaSq*i_lambda2;
					qval_p = sqrt(qval_p2)*i_lambda;
					if ((laue_l = (*lauef)(&ind_l, &cellNumC_P)) == 0.0) {
						datPtrF[pos++] = 0.0;
						datPtr[j] = 0;
						continue;
					}
					cf_h_c = cf_h_cx * qval_p;
					cf_h_s = cf_h_sx * qval_p;
					cf_h_0 = cf_h_0z * lTmp0;
					cf_k_s = cf_k_sx * qval_p;
					cf_k_0 = cf_k_0z * lTmp0;
					if (repDivFlag) {
						repNum = dToLong(qval_p*i_repDiv);
						if (repNum < REPDIV_MIN) repNum = REPDIV_MIN;
					}
					divWidth = M_2PI / repNum;
					ss = 0.0;
					for (jj=0; jj<ATOMTABLESIZE; jj++) {
						if (fbufindex[jj]) {
							if (calcXrayFactor(&qval2, &jj, &(fbuf[jj][0]))) {
								fprintf(stderr, "Error in Calculating Scattering Factor.\n");
								free(datPtrF);
								free(xyz);
								if (flushWidth0 > 0) free(datPtr0);
								if (i) {
									ipHeader.zpxl = ipHeader.rnum = i;
									if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									fseek(fp2, 0, SEEK_SET);
									fwrite(&ipHeader, ipHeaderSize, 1, fp2);
									fprintf(stdout, " Output File is saved using available data.\n");
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									if (verboseFlag) {
										fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
										fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
									}
									fclose(fp2);
								}
								else {
									fclose(fp2);
									remove(file2);
									fprintf(stderr, " Output File is not created or removed.\n");
								}
								exit(EXIT_FAILURE);
							}
							fbuf[jj][0] += fbuf[jj][1];
						}
					}
					for (ii=0; ii<repNum; ii++) {
						xi = ii * divWidth;
						sinXi = sin(xi);
						ind_h = cf_h_s * sinXi + cf_h_c * cos(xi) + cf_h_0;
						ind_k = cf_k_s * sinXi + cf_k_0;
						if ((laue_h = (*lauef)(&ind_h, &cellNumA_P)) == 0.0
						 || (laue_k = (*lauef)(&ind_k, &cellNumB_P)) == 0.0) continue;
						f_real = f_imag = 0.0;
						for (jj=0; jj<atomNum; jj++) {
							phi = M_2PI * (xyz[jj].val[0]*ind_h + xyz[jj].val[1]*ind_k + xyz[jj].val[2]*ind_l);
							cosPhi = cos(phi);
							sinPhi = sin(phi);
							f_real += fbuf[xyz[jj].atomType][0] * cosPhi - fbuf[xyz[jj].atomType][2] * sinPhi;
							f_imag += fbuf[xyz[jj].atomType][0] * sinPhi + fbuf[xyz[jj].atomType][2] * cosPhi;
						}
						sf = f_real * f_real + f_imag * f_imag;
						ss += sf * laue_h * laue_h * laue_k * laue_k;
					}
					ss *= (corr * fact2 * laue_l * laue_l * divWidth);
					datPtrF[pos++] = ss;
					if (ss > maxVal && qval2 > qlim) maxVal = ss;
					intensityDbl = ss * mag;
					datPtr[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				fflush(fp2);
				free(datPtr);
				if (debug) {
					fprintf(stdout, "\b\b\b\b\b\b\b\b");
					fprintf(stdout, "%6.2f%% ", (i+1)*100.0/ipHeader.zpxl);
					fflush(stdout);
				}
			}
		}
	}
	else {
		if (tiltFlag) {
			for (i=0; i<ipHeader.zpxl; i++) {
				zDist = (i+0.5-ipHeader.drtz) * fact1z;
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrF);
					free(xyz);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						}
						fclose(fp2);
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					xDist = (j+0.5-ipHeader.drtx) * fact1x;
					(*sinThetaf)(&xDist, &zDist, &sinThetaSq, &qval_p2, &lTmp0, &corr);
					lTmp0 *= i_lambda;	/* q_z */
					qval2 = sinThetaSq*i_lambda2;
					qval_p = sqrt(qval_p2)*i_lambda;
					cf_h_c = cf_h_cx * qval_p;
					cf_h_s = cf_h_sx * qval_p;
					cf_h_0 = cf_h_0z * lTmp0;
					cf_k_c = cf_k_cx * qval_p;
					cf_k_s = cf_k_sx * qval_p;
					cf_k_0 = cf_k_0z * lTmp0;
					cf_l_s = cf_l_sx * qval_p;
					cf_l_0 = cf_l_0z * lTmp0;
					if (repDivFlag) {
						repNum = dToLong(qval_p*i_repDiv);
						if (repNum < REPDIV_MIN) repNum = REPDIV_MIN;
					}
					divWidth = M_2PI / repNum;
					ss = 0.0;
					for (jj=0; jj<ATOMTABLESIZE; jj++) {
						if (fbufindex[jj]) {
							if (calcXrayFactor(&qval2, &jj, &(fbuf[jj][0]))) {
								fprintf(stderr, "Error in Calculating Scattering Factor.\n");
								free(datPtrF);
								free(xyz);
								if (flushWidth0 > 0) free(datPtr0);
								if (i) {
									ipHeader.zpxl = ipHeader.rnum = i;
									if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									fseek(fp2, 0, SEEK_SET);
									fwrite(&ipHeader, ipHeaderSize, 1, fp2);
									fprintf(stdout, " Output File is saved using available data.\n");
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									if (verboseFlag) {
										fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
										fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
									}
									fclose(fp2);
								}
								else {
									fclose(fp2);
									remove(file2);
									fprintf(stderr, " Output File is not created or removed.\n");
								}
								exit(EXIT_FAILURE);
							}
						}
					}
					for (ii=0; ii<repNum; ii++) {
						xi = ii * divWidth;
						cosXi = cos(xi);
						sinXi = sin(xi);
						ind_h = cf_h_c * cosXi + cf_h_s * sinXi + cf_h_0;
						ind_k = cf_k_c * cosXi + cf_k_s * sinXi + cf_k_0;
						ind_l = cf_l_s * sinXi + cf_l_0;
						if ((laue_h = (*lauef)(&ind_h, &cellNumA_P)) == 0.0
						 || (laue_k = (*lauef)(&ind_k, &cellNumB_P)) == 0.0
						 || (laue_l = (*lauef)(&ind_l, &cellNumC_P)) == 0.0) continue;
						f_real = f_imag = 0.0;
						for (jj=0; jj<atomNum; jj++) {
							phi = M_2PI * (xyz[jj].val[0]*ind_h + xyz[jj].val[1]*ind_k + xyz[jj].val[2]*ind_l);
							f_real += fbuf[xyz[jj].atomType][0] * cos(phi);
							f_imag += fbuf[xyz[jj].atomType][0] * sin(phi);
						}
						sf = f_real * f_real + f_imag * f_imag;
						ss += sf * laue_h * laue_h * laue_k * laue_k * laue_l * laue_l;
					}
					ss *= (corr * fact2 * divWidth);
					datPtrF[pos++] = ss;
					if (ss > maxVal && qval2 > qlim) maxVal = ss;
					intensityDbl = ss * mag;
					datPtr[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				fflush(fp2);
				free(datPtr);
				if (debug) {
					fprintf(stdout, "\b\b\b\b\b\b\b\b");
					fprintf(stdout, "%6.2f%% ", (i+1)*100.0/ipHeader.zpxl);
					fflush(stdout);
				}
			}
		}
		else {
			for (i=0; i<ipHeader.zpxl; i++) {
				zDist = (i+0.5-ipHeader.drtz) * fact1z;
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrF);
					free(xyz);
					if (flushWidth0 > 0) free(datPtr0);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						}
						fclose(fp2);
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					xDist = (j+0.5-ipHeader.drtx) * fact1x;
					(*sinThetaf)(&xDist, &zDist, &sinThetaSq, &qval_p2, &lTmp0, &corr);
					lTmp0 *= i_lambda;
					ind_l = lTmp0*dim_c;
					qval2 = sinThetaSq*i_lambda2;
					qval_p = sqrt(qval_p2)*i_lambda;
					if ((laue_l = (*lauef)(&ind_l, &cellNumC_P)) == 0.0) {
						datPtrF[pos++] = 0.0;
						datPtr[j] = 0;
						continue;
					}
					cf_h_c = cf_h_cx * qval_p;
					cf_h_s = cf_h_sx * qval_p;
					cf_h_0 = cf_h_0z * lTmp0;
					cf_k_s = cf_k_sx * qval_p;
					cf_k_0 = cf_k_0z * lTmp0;
					if (repDivFlag) {
						repNum = dToLong(qval_p*i_repDiv);
						if (repNum < REPDIV_MIN) repNum = REPDIV_MIN;
					}
					divWidth = M_2PI / repNum;
					ss = 0.0;
					for (jj=0; jj<ATOMTABLESIZE; jj++) {
						if (fbufindex[jj]) {
							if (calcXrayFactor(&qval2, &jj, &(fbuf[jj][0]))) {
								fprintf(stderr, "Error in Calculating Scattering Factor.\n");
								free(datPtrF);
								free(xyz);
								if (flushWidth0 > 0) free(datPtr0);
								if (i) {
									ipHeader.zpxl = ipHeader.rnum = i;
									if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									fseek(fp2, 0, SEEK_SET);
									fwrite(&ipHeader, ipHeaderSize, 1, fp2);
									fprintf(stdout, " Output File is saved using available data.\n");
									if (cpuIntelFlag) swapIpHeader(&ipHeader);
									if (verboseFlag) {
										fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
										fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
									}
									fclose(fp2);
								}
								else {
									fclose(fp2);
									remove(file2);
									fprintf(stderr, " Output File is not created or removed.\n");
								}
								exit(EXIT_FAILURE);
							}
						}
					}
					for (ii=0; ii<repNum; ii++) {
						xi = ii * divWidth;
						sinXi = sin(xi);
						ind_h = cf_h_s * sinXi + cf_h_c * cos(xi) + cf_h_0;
						ind_k = cf_k_s * sinXi + cf_k_0;
						if ((laue_h = (*lauef)(&ind_h, &cellNumA_P)) == 0.0
						 || (laue_k = (*lauef)(&ind_k, &cellNumB_P)) == 0.0) continue;
						f_real = f_imag = 0.0;
						for (jj=0; jj<atomNum; jj++) {
							phi = M_2PI * (xyz[jj].val[0]*ind_h + xyz[jj].val[1]*ind_k + xyz[jj].val[2]*ind_l);
							f_real += fbuf[xyz[jj].atomType][0] * cos(phi);
							f_imag += fbuf[xyz[jj].atomType][0] * sin(phi);
						}
						sf = f_real * f_real + f_imag * f_imag;
						ss += sf * laue_h * laue_h * laue_k * laue_k;
					}
					ss *= (corr * fact2 * laue_l * laue_l * divWidth);
					datPtrF[pos++] = ss;
					if (ss > maxVal && qval2 > qlim) maxVal = ss;
					intensityDbl = ss * mag;
					datPtr[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				fflush(fp2);
				free(datPtr);
				if (debug) {
					fprintf(stdout, "\b\b\b\b\b\b\b\b");
					fprintf(stdout, "%6.2f%% ", (i+1)*100.0/ipHeader.zpxl);
					fflush(stdout);
				}
			}
		}
	}
	free(xyz);
	fseek(fp2, ipHeader.rlng, SEEK_SET);
	mag *= (maxInt/maxVal);
	pos = 0;
	for (i=0; i<ipHeader.zpxl; i++) {
		if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtrF);
			if (flushWidth0 > 0) free(datPtr0);
			ipHeader.zpxl = ipHeader.rnum = i;
			if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
			if (cpuIntelFlag) swapIpHeader(&ipHeader);
			fseek(fp2, 0, SEEK_SET);
			fwrite(&ipHeader, ipHeaderSize, 1, fp2);
			fprintf(stdout, " Output File is saved using available data,\n");
			fprintf(stdout, "  and they are partially scalled.\n");
			if (cpuIntelFlag) swapIpHeader(&ipHeader);
			if (verboseFlag) {
				fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
			}
			fclose(fp2);
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++) {
			intensityDbl = datPtrF[pos++] * mag;
			datPtr[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
		}
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
	}
	
/* End */
	free(datPtrF);
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
		fprintf(stdout, " Output Intensity is magnified by %e times for scaling.\n", mag);
	}
	fclose(fp2);
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag ? "Little" : "Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag ? "Little" : "Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file1 {-f|-cv}\n", ProgName);
	fprintf(fp0, "          -x:VALUE -z:VALUE -dx:VALUE -dz:VALUE -cl:VALUE -lm:VALUE\n");
	fprintf(fp0, "          -a:VALUE -b:VALUE -c:VALUE -A:VALUE -B:VALUE -C:VALUE\n");
	fprintf(fp0, "          [-sc] [-m:VALUE] {-aa|-bb|-cc} [-tl:VELUE1:VALUE2] [-r] [-cq:VALUE]\n");
	fprintf(fp0, "          {-la:VALUE1:VALUE2:VALUE3|-La:VALUE1:VALUE2:VALUE3}\n");
	fprintf(fp0, "          [-nm:VALUE|-dy:VALUE] [-d] [-an] file2\n");
	fprintf(fp0, "        %s [-q|-v] file1 {-f|-cv}\n", ProgName);
	fprintf(fp0, "          {-r:REFFILE|-R:REFFILE}\n");
	fprintf(fp0, "          -a:VALUE -b:VALUE -c:VALUE -A:VALUE -B:VALUE -C:VALUE\n");
	fprintf(fp0, "          [-sc] [-m:VALUE] {-aa|-bb|-cc} [-tl:VELUE1:VALUE2] [-r] [-cq:VALUE]\n");
	fprintf(fp0, "          {-la:VALUE1:VALUE2:VALUE3|-La:VALUE1:VALUE2:VALUE3}\n");
	fprintf(fp0, "          [-nm:VALUE|-dy:VALUE] [-d] [-an] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Generate R-AXIS Image File for Fiber specimen,\n");
	fprintf(fp0, "  using atomic coordinates.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -f .......... Assume Camera is Flat.\n");
	fprintf(fp0, "   -cv ......... Assume Camera is Cylindrical, whose axis vertical to screen.\n");
	fprintf(fp0, "   -x:VALUE .... Set Number of Pixel in X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -z:VALUE .... Set Number of Pixel in Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -sc ......... Set Centre Position to ( 0.5, 0.5 ).\n");
	fprintf(fp0, "                 If absent, set to ( 0.0, 0.0 ).\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -cl:VALUE ... Set Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -lm:VALUE ... Set wavelength to \"VALUE\" [A].\n");
	fprintf(fp0, "   -r:REFFILE\n");
	fprintf(fp0, "             ... Instead of setting file parameters in above options,\n");
	fprintf(fp0, "                 use parameters just same as \"REFFILE\".\n");
	fprintf(fp0, "                 \"REFFILE\" must be valid R-AXIS Image File.\n");
	fprintf(fp0, "   -R:REFFILE\n");
	fprintf(fp0, "             ... Same as above, but uses file the largest quadrant\n");
	fprintf(fp0, "                 around the centre as a image size.\n");
	fprintf(fp0, "   -m:VALUE .... Set magnification to the calculated intensity to \"VALUE\".\n");
	fprintf(fp0, "                 Note: This is not necessary, for intensity is normalized\n");
	fprintf(fp0, "                 by default. Use this when you want to multiply moreover.\n");
	fprintf(fp0, "   -a:VALUE .... Set a-dimension to \"VALUE\".\n");
	fprintf(fp0, "   -b:VALUE .... Set b-dimension to \"VALUE\".\n");
	fprintf(fp0, "   -c:VALUE .... Set c-dimension to \"VALUE\".\n");
	fprintf(fp0, "   -A:VALUE .... Set angle alpha (between b and c) to \"VALUE\".\n");
	fprintf(fp0, "   -B:VALUE .... Set angle beta  (between c and a) to \"VALUE\".\n");
	fprintf(fp0, "   -C:VALUE .... Set angle gamma (between a and b) to \"VALUE\".\n");
	fprintf(fp0, "   -aa ......... Assume Fiber-Axis is \"a\".\n");
	fprintf(fp0, "   -bb ......... Assume Fiber-Axis is \"b\".\n");
	fprintf(fp0, "   -cc ......... Assume Fiber-Axis is \"c\".\n");
	fprintf(fp0, "                 Caution : Only 1 of above is valid.\n");
	fprintf(fp0, "   -tl:VALUE1:VALUE2\n");
	fprintf(fp0, "             ... Set tilt angle to \"VALUE2\"[deg.] around an axis, which is\n");
	fprintf(fp0, "                 lying in the plane made by reciprocal axes normal to\n");
	fprintf(fp0, "                 the fiber-axis, and making angle of \"VALUE1\"[deg.]\n");
	fprintf(fp0, "                 from one reciprocal axis which is seen as the first term\n");
	fprintf(fp0, "                 of vector product defining reciprocal fiber axis.\n");
	fprintf(fp0, "                 For example, if c- is fiber axis, tilting axis is lying\n");
	fprintf(fp0, "                 in the plane made by a*- and b*- axes, and makes angle\n");
	fprintf(fp0, "                 of \"VALUE1\"[deg.] from a*- axis toward b*- axis.\n");
	fprintf(fp0, "   -la:VALUE1:VALUE2:VALUE3\n");
	fprintf(fp0, "            .... Multiply Laue Function with Num. of cell in a, b, c equal\n");
	fprintf(fp0, "                 \"VALUE1\", \"VALUE2\", \"VALUE3\" respectively.\n");
	fprintf(fp0, "   -La:VALUE1:VALUE2:VALUE3\n");
	fprintf(fp0, "            .... Same as above, but only main maximum is used.\n");
	fprintf(fp0, "   -r .......... Grant coordinates in input file as to be fractional.\n");
	fprintf(fp0, "                 If absent, grant as to be absolute.\n");
	fprintf(fp0, "   -nm:VALUE ... Set number of division in calculating circular average.\n");
	fprintf(fp0, "                 If absent, set to %d\n", REPNUM_DEFAULT);
	fprintf(fp0, "   -dy:VALUE ... Calculate with \"VALUE\"(1/A) period on the circle to average.\n");
	fprintf(fp0, "                 At least %d times division is used for accuracy.\n", REPDIV_MIN);
	fprintf(fp0, "                 Hint : VALUE=0.01 causes 157 for d=4(A).\n");
	fprintf(fp0, "                 This is exclusive with \"-nm\" option.\n");
	fprintf(fp0, "   -d .......... Print message after 1-line.\n");
	fprintf(fp0, "   -cq:VALUE ... Ignore pixels with q smaller than \"VALUE\" while\n");
	fprintf(fp0, "                 normalization. If absent, set to %6.4f[1/A].\n", QLIM_DEFAULT);
	fprintf(fp0, "   -an ......... Anomalous scattering factor is also taken into account.\n");
	fprintf(fp0, "   file1 ....... Input X-Mol XYZ File. This must be the first argument.\n");
	fprintf(fp0, "                 If positions are in absolute coordinate values, system is\n");
	fprintf(fp0, "                 assumed to be the followings : fiber axis is along z,\n");
	fprintf(fp0, "                 preceding column to it is in yz-plane.\n");
	fprintf(fp0, "                 For example, if c- is fiber axis, c- in z-axis, b- in yz-plane.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2, arg3)
	char *argV, *arg0;
	double *arg1, *arg2, *arg3;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2, double *arg3)
#endif
{
	char *chPtr1, *chPtr2, *chPtr3;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 4) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 4) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg2 = atof(chPtr2+1);
	if ((chPtr3 = strchr(chPtr2+1, ':')) == NULL) return (3);
	if (!isdigitEXT(chPtr3+1)) return (0);
	*arg3 = atof(chPtr3+1);
	return (4);
}

#ifdef _OLDHP
char rotate3Dbl(arg1, arg2, arg3)
	double *arg1, *arg2, *arg3;
#else
char rotate3Dbl(double *arg1, double *arg2, double *arg3)
#endif
{
	double dBuf;
	
	dBuf = *arg1;
	*arg1 = *arg2;
	*arg2 = *arg3;
	*arg3 = dBuf;
	return (0);
}

#ifdef _OLDHP
char rotate3Long(arg1, arg2, arg3)
	long int *arg1, *arg2, *arg3;
#else
char rotate3Long(long int *arg1, long int *arg2, long int *arg3)
#endif
{
	long int lBuf;
	
	lBuf = *arg1;
	*arg1 = *arg2;
	*arg2 = *arg3;
	*arg3 = lBuf;
	return (0);
}

#ifdef _OLDHP
char sinThetaFlat(x, z, sinThetaSq, qval_p2, zeta, corr)
	double *x, *z, *sinThetaSq, *qval_p2, *zeta, *corr;
#else
char sinThetaFlat(const double *x, const double *z, double *sinThetaSq, double *qval_p2, double *zeta, double *corr)
#endif
{
	double denom, denomS, xsq, zsq, numer;
	/* z = z/camr , x = x/camr */
	xsq = (*x) * (*x);
	zsq = (*z) * (*z);
	numer = xsq + zsq;
	denom = numer + 1.0;
	denomS = sqrt(denom);
	*qval_p2 = (4.0*xsq + zsq*zsq/denom) / (denom + 2.0*denomS + xsq + 1.0);
	*sinThetaSq = numer*2.0/(denomS+denom); /* 2*sin(theta) */
	*zeta = fabs(*z) / denomS;
	*corr = 1.0 / (denom*denomS);
	return (0);
}

#ifdef _OLDHP
char sinThetaCurveV(x, z, sinThetaSq, qval_p2, zeta, corr)
	double *x, *z, *sinThetaSq, *qval_p2, *zeta, *corr;
#else
char sinThetaCurveV(const double *x, const double *z, double *sinThetaSq, double *qval_p2, double *zeta, double *corr)
#endif
{
	double denom, denomS, xsqn, zsqn, cx, sinx;
	/* z = z/camr , x = x/camr */
	zsqn = (*z) * (*z);
	denom = zsqn + 1.0;
	denomS = sqrt(denom);
	sinx = sin(*x);
	xsqn = sinx * sinx;
	if ((cx = cos(*x)) < 0.0) {
		*qval_p2 = 2.0 - 2.0*cx/denomS - zsqn/denom;
		*sinThetaSq = 2.0 - 2.0*cx/denomS;
	}
	else {
		*qval_p2 = (4.0*xsqn + zsqn*zsqn/denom) / (2.0*(denom+denomS*cx) - zsqn);
		*sinThetaSq = (zsqn+xsqn)*2.0/(denom+denomS*cx);
	}
	*corr = 1.0 / (denom*denomS);
	*zeta = fabs(*z) / denomS;
	return (0);
}

#ifdef _OLDHP
double lauefunc(ind, size)
	double *ind, *size;
#else
double lauefunc(double *ind, double *size)
#endif
{
	if (fabs(fmod((*ind), 1.0)) == 0.0) return ((*size)/M_PI);
	return (sin((*ind)*(*size))/sin((*ind)*M_PI));
}

#ifdef _OLDHP
double lauefunc2(ind, size)
	double *ind, *size;
#else
double lauefunc2(double *ind, double *size)
#endif
{
	double res;
	
	if ((res = fabs(fmod((*ind), 1.0))) == 0.0) return ((*size)/M_PI);
	if (res*(*size) < M_PI || (1.0-res)*(*size) < M_PI) return (sin((*ind)*(*size))/sin((*ind)*M_PI));
	return (0.0);
}
